@@ -4,6 +4,7 @@ import os
 import hashlib
 import subprocess
 import inspect
+import json
 from collections import defaultdict
 from dbm import dumb
 import dbm as ddbm
@@ -16,7 +17,6 @@ import dbm as ddbm
 # note: to check which DBM backend is being used (in py2):
 #       >>> anydbm._defaultmod
 
-import json
 
 
 class DatabaseException(Exception):
@@ -47,12 +47,27 @@ def get_file_md5(path):
     return md5.hexdigest()
 
 
+class JSONCodec():
+    """default implmentation for codec used to save individual task's data"""
+    def __init__(self):
+        self.encoder = json.JSONEncoder()
+        self.decoder = json.JSONDecoder()
+
+    def encode(self, data):
+        return self.encoder.encode(data)
+
+    def decode(self, data):
+        return self.decoder.decode(data)
+
+
+
 class JsonDB(object):
     """Backend using a single text file with JSON content"""
 
-    def __init__(self, name):
+    def __init__(self, name, codec):
         """Open/create a DB file"""
         self.name = name
+        self.codec = codec
         if not os.path.exists(self.name):
             self._db = {}
         else:
@@ -63,7 +78,7 @@ class JsonDB(object):
         db_file = open(self.name, 'r')
         try:
             try:
-                return json.load(db_file)
+                return self.codec.decode(db_file.read())
             except ValueError as error:
                 # file contains corrupted json data
                 msg = (error.args[0] +
@@ -80,7 +95,7 @@ class JsonDB(object):
         """save DB content in file"""
         try:
             db_file = open(self.name, 'w')
-            json.dump(self._db, db_file)
+            db_file.write(self.codec.encode(self._db))
         finally:
             db_file.close()
 
@@ -119,22 +134,24 @@ class JsonDB(object):
 class DbmDB(object):
     """Backend using a DBM file with individual values encoded in JSON
 
-    On initialization all items are read from DBM file and loaded on _dbm.
-    During execution whenever an item is read ('get' method) the json value
-    is cached on _db. If a item is modified _db is update and the id is added
-    to the 'dirty' set. Only on 'dump' all dirty items values are encoded
-    in json into _dbm and the DBM file is saved.
+    On initialization all items are read from DBM file and loaded on ``_dbm``.
+    During execution whenever an item is read (``get`` method) the `json` value
+    is cached on ``_db``.
+    If an item is modified ``_db`` is update and the `id` is added
+    to the `dirty` set. Only on ``dump`` all dirty items values are encoded
+    in json into ``_dbm`` and the DBM file is saved.
 
-    @ivar name: (str) file name/path
-    @ivar _dbm: (dbm) items with json encoded values
-    @ivar _db: (dict) items with python-dict as value
-    @ivar dirty: (set) id of modified tasks
+    :ivar str name: file name/path
+    :ivar dbm _dbm: items with json encoded values
+    :ivar dict _db: items with python-dict as value
+    :ivar set dirty: id of modified tasks
     """
     DBM_CONTENT_ERROR_MSG = 'db type could not be determined'
 
-    def __init__(self, name):
+    def __init__(self, name, codec):
         """Open/create a DB file"""
         self.name = name
+        self.codec = codec
         try:
             self._dbm = ddbm.open(self.name, 'c')
         except ddbm.error as exception:
@@ -159,7 +176,7 @@ class DbmDB(object):
     def dump(self):
         """save/close DBM file"""
         for task_id in self.dirty:
-            self._dbm[task_id] = json.dumps(self._db[task_id])
+            self._dbm[task_id] = self.codec.encode(self._db[task_id])
         self._dbm.close()
 
 
@@ -184,7 +201,7 @@ class DbmDB(object):
     def get(self, task_id, dependency):
         """Get value stored in the DB.
 
-        @return: (string) or (None) if entry not found
+        :return: string or None if entry not found
         """
         # optimization, just try to get it without checking it exists
         if task_id in self._db:
@@ -194,7 +211,7 @@ class DbmDB(object):
                 task_data = self._dbm[task_id]
             except KeyError:
                 return
-            self._db[task_id] = json.loads(task_data.decode('utf-8'))
+            self._db[task_id] = self.codec.decode(task_data.decode('utf-8'))
             return self._db[task_id].get(dependency, None)
 
 
@@ -231,14 +248,14 @@ class DbmDB(object):
 class SqliteDB(object):
     """ sqlite3 json backend """
 
-    def __init__(self, name):
+    def __init__(self, name, codec):
         self.name = name
+        self.codec = codec
         self._conn = self._sqlite3(self.name)
         self._cache = {}
         self._dirty = set()
 
-    @staticmethod
-    def _sqlite3(name):
+    def _sqlite3(self, name):
         """Open/create a sqlite3 DB file"""
 
         # Import sqlite here so it's only imported when required
@@ -250,10 +267,10 @@ class SqliteDB(object):
                 data[col[0]] = row[idx]
             return data
         def converter(data):
-            return json.loads(data.decode('utf-8'))
+            return self.codec.decode(data.decode('utf-8'))
 
-        sqlite3.register_adapter(list, json.dumps)
-        sqlite3.register_adapter(dict, json.dumps)
+        sqlite3.register_adapter(list, self.codec.encode)
+        sqlite3.register_adapter(dict, self.codec.encode)
         sqlite3.register_converter("json", converter)
         conn = sqlite3.connect(
             name,
@@ -314,7 +331,7 @@ class SqliteDB(object):
         """save/close sqlite3 DB file"""
         for task_id in self._dirty:
             self._conn.execute('insert or replace into doit values (?,?)',
-                               (task_id, json.dumps(self._cache[task_id])))
+                               (task_id, self.codec.encode(self._cache[task_id])))
         self._conn.commit()
         self._conn.close()
         self._dirty = set()
@@ -337,6 +354,14 @@ class SqliteDB(object):
 class FileChangedChecker(object):
     """Base checker for dependencies, must be inherited."""
 
+    CheckerError = os.error
+
+    def exists(self, file_path):
+        return os.path.exists(file_path)
+
+    def info(self, file_path):
+        return os.stat(file_path)
+
     def check_modified(self, file_path, file_stat, state):
         """Check if file in file_path is modified from previous "state".
 
@@ -455,24 +480,35 @@ class DependencyStatus(object):
 
 
 class Dependency(object):
-    """Manage tasks dependencies
+    """Manage tasks dependencies.
 
-    Each dependency is saved in "db". the "db" can have json or dbm
-    format where there is a dictionary for every task. each task has a
-    dictionary where key is a dependency (abs file path), and the value is the
-    dependency signature.
-    Apart from dependencies other values are also saved on the task dictionary
-     * 'result:', 'task:<task-name>', 'ignore:'
-     * user(task) defined values are defined in '_values_:' sub-dict
+    Each dependency is saved in "db". There are several "db" backends.
+    It uses a Key-Value format where the key is task-name
+    and value is a dictionary.
+    Each task has a dictionary where keys are `dependency`'s (absolute file path),
+    and the value is the dependency signature.
+    Apart from dependencies other values are also saved on the task dictionary:
 
-    @ivar name: (string) filepath of the DB file
-    @ivar _closed: (bool) DB was flushed to file
+    * ``_values_:`` task's values
+    * ``result:`` task result
+
+    And also some internal doit attributes:
+
+    * ``ignore:``
+    * ``deps:``
+    * ``checker:``
+
+    Those can be accessed with generic DB ``get()``, see below...
+
+    :ivar string name: filepath of the DB file
+    :ivar bool _closed: DB was flushed to file
     """
-    def __init__(self, db_class, backend_name, checker_cls=MD5Checker):
+    def __init__(self, db_class, backend_name, checker_cls=MD5Checker,
+                 codec_cls=JSONCodec):
         self._closed = False
         self.checker = checker_cls()
         self.db_class = db_class
-        self.backend = db_class(backend_name)
+        self.backend = db_class(backend_name, codec=codec_cls())
         self._set = self.backend.set
         self._get = self.backend.get
         self.remove = self.backend.remove
@@ -492,7 +528,7 @@ class Dependency(object):
     def save_success(self, task, result_hash=None):
         """save info after a task is successfully executed
 
-        :param result_hash: (str) explicitly set result_hash
+        :param str result_hash: explicitly set result_hash
         """
         # save task values
         self._set(task.name, "_values_:", task.values)
@@ -518,15 +554,17 @@ class Dependency(object):
 
     def get_values(self, task_name):
         """get all saved values from a task
-        @return dict
+
+        :return dict:
         """
         values = self._get(task_name, '_values_:')
         return values or {}
 
     def get_value(self, task_id, key_name):
         """get saved value from task
-        @param task_id (str)
-        @param key_name (str): key result dict of the value
+
+        :param str task_id:
+        :param str key_name: key result dict of the value
         """
         if not self._in(task_id):
             # FIXME do not use generic exception
@@ -539,7 +577,8 @@ class Dependency(object):
 
     def get_result(self, task_name):
         """get the result saved from a task
-        @return dict or md5sum
+
+        :return (dict or md5sum):
         """
         return self._get(task_name, 'result:')
 
@@ -626,7 +665,7 @@ class Dependency(object):
 
         # if target file is not there, task is not up to date
         for targ in task.targets:
-            if not os.path.exists(targ):
+            if not self.checker.exists(targ):
                 task.dep_changed = list(task.file_dep)
                 if result.add_reason('missing_target', targ):
                     return result
@@ -659,8 +698,8 @@ class Dependency(object):
         for dep in task.file_dep:
             state = self._get(task.name, dep)
             try:
-                file_stat = os.stat(dep)
-            except OSError:
+                file_stat = self.checker.info(dep)
+            except self.checker.CheckerError:
                 error_msg = "Dependent file '{}' does not exist.".format(dep)
                 result.error_reason = error_msg.format(dep)
                 if result.add_reason('missing_file_dep', dep, 'error'):
